#include <qapplication.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qpainter.h>
#include <iostream.h>
#include "gmainwindow.h"


//! Constructor for the GMainWindow class
/*! Generates the menus. */
GMainWindow::GMainWindow()
{
  imshow = new GImShow(this);
  paint = new QPainter(this);

  _filemenu = new QPopupMenu;
  openID = _filemenu->insertItem("&Open...", this,  SLOT(slotOpenFile()));
  saveID = _filemenu->insertItem("&Save...", this,  SLOT(slotSaveFile()));
  quitID = _filemenu->insertItem("&Quit", qApp, SLOT(quit()));
  setFileMenuItems(TRUE, FALSE, TRUE);
 
  _viewmenu = new QPopupMenu;
  origimageID = _viewmenu->insertItem("&Original image", imshow,  SLOT(slotViewOrigImage()));
  filterrealID = _viewmenu->insertItem("&Real filter", imshow,  SLOT(slotViewFilterReal()));
  filterimagID = _viewmenu->insertItem("&Imag filter", imshow,  SLOT(slotViewFilterImag()));
  filtermagID = _viewmenu->insertItem("&Magnitude filter", imshow,  SLOT(slotViewFilterMag()));
  resultrealID = _viewmenu->insertItem("&Real result", imshow,  SLOT(slotViewResultReal()));
  resultimagID = _viewmenu->insertItem("&Imag result", imshow,  SLOT(slotViewResultImag()));
  resultmagID = _viewmenu->insertItem("&Magnitude result", imshow,  SLOT(slotViewResultMag()));
  setViewMenuItems(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE);

  _settingsmenu = new QPopupMenu;
  filterID = _settingsmenu->insertItem("&Filter settings...", this,  SLOT(slotOpenGDialog()));
  setSettingsMenuItems(FALSE);

  _helpmenu = new QPopupMenu;
  _helpmenu->insertItem("&About QtGabor2D", this, SLOT(slotAboutQtGabor2D()));
 
  _menubar = new QMenuBar(this);
  _menubar->insertItem("&File", _filemenu);
  _menubar->insertItem("&View", _viewmenu);
  _menubar->insertItem("&Settings", _settingsmenu);
  _menubar->insertSeparator();
  _menubar->insertItem("&Help", _helpmenu);    
                                
  imshow->show();

  dialog = new GDialog(this,0,0);

  connect(dialog, SIGNAL(sendParam(QString,QString, QString, QString, QString, QString, QString, QString, QString, QString)), imshow, SLOT(slotSetFilter(QString,QString, QString, QString, QString, QString, QString, QString, QString, QString) ) );
  connect(dialog, SIGNAL(sendParam(QString,QString, QString, QString, QString, QString, QString, QString, QString, QString)), this, SLOT(slotFullViewMenu() ) 
);
}

GMainWindow::~GMainWindow()
{
}

void GMainWindow::paintEvent(QPaintEvent *e)
{
  paint->setClipRect(e->rect());
  resize(imshow->width(),imshow->height()+_menubar->heightForWidth(imshow->height()));
  imshow->repaint();
}


//! Simple dialogbox on the Help menu
void GMainWindow::slotAboutQtGabor2D()
{
  QMessageBox::information(this, "About QtGabor2D", "by Erik Hjelms\n");
} 


void GMainWindow::slotOpenFile()
{
  QString newfilename = QFileDialog::getOpenFileName();
  if(!newfilename.isEmpty()) 
  {
    imshow->loadImage(newfilename);
    imshow->setGeometry(0,_menubar->heightForWidth(imshow->height()),imshow->width(),imshow->height());
    resize(imshow->width(),imshow->height()+_menubar->heightForWidth(imshow->height()));
    imshow->update();
    setFileMenuItems(TRUE, TRUE, TRUE);
    setViewMenuItems(TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE);
    setSettingsMenuItems(TRUE);
  }
  else  
    cout << "ikke innlest fil!" << endl;
}

void GMainWindow::slotSaveFile()
{
  QString savefilename = QFileDialog::getSaveFileName("", "", this);
  if (!savefilename.isEmpty()) 
    imshow->saveImage(savefilename);
  else
    cout << "ikke lagret til fil!" << endl;
}

void GMainWindow::slotOpenGDialog()
{
  dialog->show();
}

void GMainWindow::slotFullViewMenu()
{
  setViewMenuItems(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE);
}

void GMainWindow::setFileMenuItems(bool open, bool save, bool quit)
{
  _filemenu->setItemEnabled(openID, open);
  _filemenu->setItemEnabled(saveID, save);
  _filemenu->setItemEnabled(quitID, quit);
}

void GMainWindow::setViewMenuItems(bool origimage, bool filterreal, bool filterimag, bool filtermag, bool resultreal, bool resultimag, bool resultmag)
{
  _viewmenu->setItemEnabled(origimageID, origimage);
  _viewmenu->setItemEnabled(filterrealID, filterreal);
  _viewmenu->setItemEnabled(filterimagID, filterimag);
  _viewmenu->setItemEnabled(filtermagID, filtermag);
  _viewmenu->setItemEnabled(resultrealID, resultreal);
  _viewmenu->setItemEnabled(resultimagID, resultimag);
  _viewmenu->setItemEnabled(resultmagID, resultmag);
}

void GMainWindow::setSettingsMenuItems(bool filter)
{
  _settingsmenu->setItemEnabled(filterID, filter);
}















