#ifndef GMAINWINDOW_H
#define GMAINWINDOW_H

#include <qmainwindow.h>                       
#include "gimshow.h"
#include "gdialog.h"
#include <qpainter.h>

//! The main class for the QtGabor2D application
/*! This class owns the menubar and the qimage displayed. By using the menus, the user can control what is displayed. The file menu lets the user open and save images, the view menu controls the viewer, but the most important functionality is in the settings menu, where the user can set the parameters for the Gabor filters. */
class GMainWindow : public QMainWindow
{
  Q_OBJECT // keyword for meta object compiler

 private:
  QPainter *paint;
  GImShow *imshow;
  GDialog *dialog;
  QMenuBar *_menubar;
  QPopupMenu *_filemenu;
  QPopupMenu *_helpmenu;
  QPopupMenu *_viewmenu;
  QPopupMenu *_settingsmenu;
  int openID, saveID, quitID, origimageID, filterrealID, filterimagID, filtermagID, resultrealID, resultimagID, resultmagID, filterID;

 private slots:
  void slotAboutQtGabor2D();
  void slotOpenFile();
  void slotSaveFile();
  void slotOpenGDialog();
  void slotFullViewMenu();

 protected:
  void paintEvent(QPaintEvent *e);

 private:
  void setFileMenuItems(bool open, bool save, bool quit);
  void setViewMenuItems(bool origimage, bool filterreal, bool filterimag, bool filtermag, bool resultreal, bool resultimag, bool resultmag);
  void setSettingsMenuItems(bool filter);

 public:
  GMainWindow();
  ~GMainWindow();

};


#endif
