#ifndef _GRAYPROCESSINGIMAGE_H
#define _GRAYPROCESSINGIMAGE_H

#include <vector>
#include <cmath>
#include <qimage.h>

//! This class defines various image processing operations.
/*! The class derives from the QImage class. When the image is
    instantiated, it is converted to gray scale once and for all. */
class GrayProcessingImage : public QImage
{
private:
  void fillGrayPalette();
  void toGrayscale();
  void calculateHistogram(unsigned long *histogram);
  void calculateIntensityPlane(double *a, double *b, double *c);
  uchar interpolate(double x, double y);
public:
  //! Constructors
  GrayProcessingImage();
  GrayProcessingImage(QImage image);
  GrayProcessingImage(const GrayProcessingImage& image);
  GrayProcessingImage(int width, int height, int depth=8, int numColors=256,
		      Endian bitOrder=IgnoreEndian );
  GrayProcessingImage(const QSize& size, int depth = 8, int numColors=256,
		      Endian bitOrder=IgnoreEndian);

  //! Handy stuff
  enum PaddingType { zeroPadding, copyPadding, wrapPadding };
  enum ImageType { realImage, imagImage, magImage };
  bool load(const QString& fileName, const char *format=0);

  //! Image operations
  GrayProcessingImage bilinearResize(int h, int w);
  GrayProcessingImage intensityPlane();
  GrayProcessingImage subtractIntensityPlane();
  GrayProcessingImage histogramEqualize();
  GrayProcessingImage convolution3x3(double matrix[3][3], int add = 0,
		      PaddingType p = zeroPadding);
  GrayProcessingImage average(vector<GrayProcessingImage> images);
  GrayProcessingImage faceNormalize(int lx, int ly, int rx, int ry,
				    int pctabove = 30, int pctofwidth = 60,
				    int neww = 20, int newh = 20);

  GrayProcessingImage normal1();
  GrayProcessingImage normal2();
  pair<GrayProcessingImage, GrayProcessingImage> dirImage(int windowSize);
  GrayProcessingImage visualiseDirImage(int windowSize);
  GrayProcessingImage visualiseDirImage(pair<GrayProcessingImage,
					GrayProcessingImage> dir);
  double average();
  double variance();
  double correlcoeff(GrayProcessingImage image);

  void setImage(complex<double> **img, int w, int h, double max, double min, int i);
};

#endif //  _GRAYPROCESSINGIMAGE_H
